import {
  GridEditable, Button, IComponentRender, DialogService,
} from '@zeedhi/common';
import {
  Loader, Metadata, IDatasource, IEventParam, IDictionary,
} from '@zeedhi/core';
import { DragDropGrid } from '@/components/drag-drop-grid/common/DragDropGrid';

export class GridEditableController {
  public crudController = Loader.getInstance('CrudController');

  public utilities = Loader.getInstance('Utilities');

  public isInlineEditing = false;

  public newRow = false;

  private checkedRows!: IDictionary;

  private grid!: GridEditable;

  private datasourceName = '';

  public toggleDense({ component }: IEventParam<Button>) {
    (component.parent as GridEditable).dense = !(component.parent as GridEditable).dense;
  }

  public reload({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    grid.datasource.get();
    if (this.isInlineEditing) this.defaultState(grid);
  }

  public onInlineEdit({ component }: IEventParam<GridEditable>) {
    const toolbarSlot = component.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = !e.disabled;
      if (e.name === 'saveButton') e.disabled = !e.disabled;
    });
    this.isInlineEditing = component.editing;
  }

  public dragOnInlineEdit({ component }: IEventParam<DragDropGrid>) {
    const toolbarSlot = component.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = false;
      if (e.name === 'saveButton') e.disabled = false;
    });
  }

  public get editing() {
    return true;
  }

  public cancelEditing({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    if (grid.editing) this.defaultState(grid);
  }
  
  public addRow({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;

    if (!this.newRow) {
      grid.datasource.data.unshift({ __is_new: true });
      this.newRow = true;
      if (!grid.editing) {
        grid.editing = true;
        if (!this.isInlineEditing) this.onInlineEdit({ component: grid });
      }
    }
  }

  public async saveInlineEdit({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    
    const editedRows = grid.getEditedRows();
    const regexFind = /\/find/gm;
    const datasourceName = (grid.datasource as IDatasource).route.replace(regexFind, '');

    this.crudController.saveRows(editedRows, datasourceName).then(() => {
      this.defaultState(grid);
      this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }

  public defaultState(grid: GridEditable) {
    this.newRow = false;
    grid.cancelEditedRows();
    this.onInlineEdit({ component: grid });
    return grid.datasource.get();
  }

  public async deleteCheckedRows({ component }: IDictionary) {
    this.grid = this.utilities.returnGrid(component) as GridEditable;
    this.checkedRows = this.grid.selectedRows;
    if (this.checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      const regexFind = /\/find/gm;
      this.datasourceName = (this.grid.datasource as IDatasource).route.replace(regexFind, '');
      DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.callDeleteRows() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }

  private callDeleteRows() {
    this.crudController.deleteRows(this.checkedRows, this.datasourceName).then(() => {
      this.grid.cancelEditedRows();
      this.grid.datasource.get();
      this.grid.selectedRows = [];
      DialogService.hide();
      this.utilities.showAlert('alert', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      DialogService.hide();
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }

  public addGroupArrow(gridOriName: string, gridDestName: string) {
    const gridOri: GridEditable = Metadata.getInstance(gridOriName);
    const gridDest: GridEditable = Metadata.getInstance(gridDestName);
    const currentRow = gridOri.datasource.currentRow;
    if (currentRow) {
      currentRow.__is_new = true;
      gridDest.datasource.data.unshift(currentRow);
      this.newRow = true;
      if (!gridDest.editing) {
        gridDest.editing = true;
        this.onInlineEdit({ component: gridDest });
      }
    }
  }

  public addRowArrow(currentRow: { __is_new: true }, gridDestName: string) {
    const gridDest: GridEditable = Metadata.getInstance(gridDestName);
    if (currentRow) {
      currentRow.__is_new = true;
      // gridDest.addNewRow(currentRow);
      gridDest.datasource.data.push(currentRow);
      if (!this.newRow) {
        this.newRow = true;
        if (!gridDest.editing) {
          gridDest.editing = true;
          this.onInlineEdit({ component: gridDest });
        }
      }
    }
  }
  
  public editRow({ row }: IDictionary) {
    return row.__is_new;
  }

  public resetController() {
    this.newRow = false;
    this.isInlineEditing = false;
    this.datasourceName = '';
    this.checkedRows = [];
  }
}
